@@ -12,7 +12,7 @@ use HTML::Parser 3.26;
 require 5.005;
 
 use vars qw($VERSION @ISA);
-$VERSION = '1.04';
+$VERSION = '1.07';
 
 @ISA = qw(HTML::Parser);
 
@@ -38,9 +38,15 @@ sub fill {
     ? @{ $option{ignore_fields} } : $option{ignore_fields} if exists( $option{ignore_fields} );
   $self->{ignore_fields} = \%ignore_fields;
 
+  my %disable_fields;
+  %disable_fields = map { $_ => 1 } ( ref $option{'disable_fields'} eq 'ARRAY' )
+    ? @{ $option{disable_fields} } : $option{ignore_fields} if exists( $option{disable_fields} );
+  $self->{disable_fields} = \%disable_fields;
+
   if (my $fdat = $option{fdat}){
     # Copy the structure to prevent side-effects.
     my %copy;
+    keys %$fdat; # reset fdat if each or Dumper was called on fdat
     while(my($key, $val) = each %$fdat) {
       next if exists $ignore_fields{$key};
       $copy{ $key } = ref $val eq 'ARRAY' ? [ @$val ] : $val;
@@ -89,6 +95,8 @@ sub fill {
       $self->parse($_);
     }
   }
+
+  $self->eof;
   return delete $self->{output};
 }
 
@@ -120,6 +128,13 @@ sub start {
     $self->{output} .= '>';
     delete $self->{option_no_value};
   }
+
+  # Check if we need to disable  this field
+  $attr->{disable} = 1
+    if exists $attr->{'name'} and
+    exists $self->{disable_fields}{ $attr->{'name'} } and
+    $self->{disable_fields}{ $attr->{'name'} } and
+    not ( exists $attr->{disable} and $attr->{disable} );
   if ($tagname eq 'input'){
     my $value = exists $attr->{'name'} ? $self->_get_param($attr->{'name'}) : undef;
     # force hidden fields to have a value
@@ -129,13 +144,22 @@ sub start {
       # check for input type, noting that default type is text
       if (!exists $attr->{'type'} ||
 	  $attr->{'type'} =~ /^(text|textfield|hidden|)$/i){
-	$value = (shift @$value || '') if ref($value) eq 'ARRAY';
+	if ( ref($value) eq 'ARRAY' ) {
+	  $value = shift @$value;
+	  $value = '' unless defined $value;
+        }
 	$attr->{'value'} = $value;
       } elsif (lc $attr->{'type'} eq 'password' && $self->{fill_password}) {
-	$value = (shift @$value || '') if ref($value) eq 'ARRAY';
+	if ( ref($value) eq 'ARRAY' ) {
+	  $value = shift @$value;
+	  $value = '' unless defined $value;
+        }
 	$attr->{'value'} = $value;
       } elsif (lc $attr->{'type'} eq 'radio'){
-	$value = ($value->[0] || '') if ref($value) eq 'ARRAY';
+	if ( ref($value) eq 'ARRAY' ) {
+	  $value = $value->[0];
+	  $value = '' unless defined $value;
+        }
 	# value for radio boxes default to 'on', works with netscape
 	$attr->{'value'} = 'on' unless exists $attr->{'value'};
 	if ($attr->{'value'} eq $value){
@@ -168,18 +192,31 @@ sub start {
     $self->{output} .= ">";
   } elsif ($tagname eq 'option'){
     my $value = $self->_get_param($self->{selectName});
+    $value = [ $value ] unless ( ref($value) eq 'ARRAY' );
 
-    if (defined($value)){
+    if ( defined $value->[0] ){
       $value = $self->escapeHTMLStringOrList($value);
-      $value = [ $value ] unless ( ref($value) eq 'ARRAY' );
       delete $attr->{selected} if exists $attr->{selected};
-
+      
       if(defined($attr->{'value'})){
         # option tag has value attr - <OPTION VALUE="foo">bar</OPTION>
-	foreach my $v ( @$value ) {
-	  if ( $attr->{'value'} eq $v ) {
-	    $attr->{selected} = 'selected';
-	  }
+        
+        if ($self->{selectMultiple}){
+          # check if the option tag belongs to a multiple option select
+	  foreach my $v ( grep { defined } @$value ) {
+	    if ( $attr->{'value'} eq $v ){
+	      $attr->{selected} = 'selected';
+	    }
+          }
+        } else {
+          # if not every value of a fdat ARRAY belongs to a different select tag
+          if (not $self->{selectSelected}){
+	    if ( $attr->{'value'} eq $value->[0]){
+	      shift @$value if ref($value) eq 'ARRAY';
+	      $attr->{selected} = 'selected';
+              $self->{selectSelected} = 1; # remeber that an option tag is selected for this select tag
+	    }
+          }
         }
       } else {
         # option tag has no value attr - <OPTION>bar</OPTION>
@@ -208,6 +245,12 @@ sub start {
     }
   } elsif ($tagname eq 'select'){
     $self->{selectName} = $attr->{'name'};
+    if (defined $attr->{'multiple'}){
+      $self->{selectMultiple} = 1; # helper var to remember if the select tag has the multiple attr set or not
+    } else {
+      $self->{selectMultiple} = 0;
+      $self->{selectSelected} = 0; # helper var to remember if an option was already selected in the current select tag
+    }
     $self->{output} .= $origtext;
   } else {
     $self->{output} .= $origtext;
@@ -303,8 +346,25 @@ sub escapeHTML {
 }
 
 sub comment {
-  my ( $self, $text ) = @_;
-  $self->{output} .= '<!--' . $text . '-->';
+    my ( $self, $text ) = @_;
+    # if it begins with '[if ' and doesn't end with '<![endif]'
+    # it's a "downlevel-revealed" conditional comment (stupid IE)
+    # or
+    # if it ends with '[endif]' then it's the end of a
+    # "downlevel-revealed" conditional comment
+    if(
+        (
+            ( index($text, '[if ') == 0 )
+            &&
+            ( $text !~ /<!\[endif\]$/ )
+        )
+        ||
+        ( $text eq '[endif]' )
+    ) {
+        $self->{output} .= '<!' . $text . '>';
+    } else {
+        $self->{output} .= '<!--' . $text . '-->';
+    }
 }
 
 sub process {
@@ -323,11 +383,12 @@ __END__
 
 =head1 NAME
 
-HTML::FillInForm - Populates HTML Forms with CGI data.
+HTML::FillInForm - Populates HTML Forms with data.
 
 =head1 DESCRIPTION
 
-This module automatically inserts data from a previous HTML form into the HTML input, textarea and select tags.
+This module automatically inserts data from a previous HTML form into the HTML input, textarea,
+radio buttons, checkboxes and select tags.
 It is a subclass of L<HTML::Parser> and uses it to parse the HTML and insert the values into the form tags.
 
 One useful application is after a user submits an HTML form without filling out a
@@ -347,6 +408,8 @@ in C<$q>.  For example, it will set the value of any "name" textfield to "John S
   my $output = $fif->fill(scalarref => \$html,
 			  fobject => $q);
 
+Note CGI.pm is B<not> required - see using fdat below.  Also you can use a CGI.pm-like object such as Apache::Request.
+
 =head1 METHODS
 
 =over 4
@@ -413,6 +476,12 @@ To disable the filling of some fields, use the C<ignore_fields> option:
                        fobject => $q,
                        ignore_fields => ['prev','next']);
 
+To disable the form from being edited, use the C<disable_fields> options:
+
+  $output = $fif->fill(scalarref => \$html,
+                       fobject => $q,
+                       disable_fields => [ 'uid', 'gid' ]);
+
 Note that this module does not clear fields if you set the value to undef.
 It will clear fields if you set the value to an empty array or an empty string.  For example:
 
@@ -449,11 +518,11 @@ HTML::FillInForm is now integrated with Apache::ASP.  To activate, use
 
 Using HTML::FillInForm from HTML::Mason is covered in the FAQ on
 the masonhq.com website at
-L<http://www.masonhq.com/docs/faq/#how_can_i_integrate_html_fillin>
+L<http://www.masonhq.com/?FAQ:HTTPAndHTML#h-how_can_i_populate_form_values_automatically_>
 
 =head1 VERSION
 
-This documentation describes HTML::FillInForm module version 1.04.
+This documentation describes HTML::FillInForm module version 1.06.
 
 =head1 SECURITY
 
@@ -486,16 +555,16 @@ into HTML forms,
 but without combining the HTML and Perl code.  CGI.pm and Embperl allow you so
 insert CGI data into forms, but require that you mix HTML with Perl.
 
+There is a nice review of the module available here:
+L<http://www.perlmonks.org/index.pl?node_id=274534>
+
 =head1 AUTHOR
 
-(c) 2004 TJ Mather, Maxmind LLC, tjmather@maxmind.com
+(c) 2005 TJ Mather, tjmather@maxmind.com, L<http://www.maxmind.com/>
 
 All rights reserved. This package is free software; you can
 redistribute it and/or modify it under the same terms as Perl itself.
 
-Paid support is available from directly from the author of this package.
-Please see L<http://www.maxmind.com/app/opensourceservices> for more details.
-
 =head1 SEE ALSO
 
 L<HTML::Parser>, L<Data::FormValidator>, L<HTML::Template>, L<Apache::PageKit>
@@ -525,5 +594,11 @@ Fixes, Bug Reports, Docs have been generously provided by:
   Bill Moseley
   James Tolley
   Dan Kubb
+  Alexander Hartmaier
+  Paul Miller
+  Anthony Ettinger
+  Simon P. Ditner
+  Michael Peters
+  Trevor Schellhorn
 
 Thanks!
